@@ -8,6 +8,7 @@ from botocore.exceptions import ClientError
 import logging
 from multiprocessing.pool import ThreadPool
 import os
+import re
 from redo import retry
 import sys
 import mimetypes
@@ -16,22 +17,28 @@ from scriptworker import client
 from scriptworker.exceptions import ScriptWorkerTaskException, ScriptWorkerRetryException
 from scriptworker.utils import retry_async, raise_future_exceptions
 
+from beetmoverscript import task, zip, maven_utils
+
 from beetmoverscript.constants import (
     MIME_MAP, RELEASE_BRANCHES, CACHE_CONTROL_MAXAGE, RELEASE_EXCLUDE,
-    TEMPLATE_KEY_PLATFORMS, NORMALIZED_BALROG_PLATFORMS
+    NORMALIZED_BALROG_PLATFORMS, PARTNER_REPACK_PUBLIC_PREFIX_TMPL,
+    PARTNER_REPACK_PRIVATE_REGEXES, PARTNER_REPACK_PUBLIC_REGEXES, BUILDHUB_ARTIFACT,
+    INSTALLER_ARTIFACTS, DEFAULT_ZIP_MAX_FILE_SIZE_IN_MB
 )
 from beetmoverscript.task import (
     validate_task_schema, add_balrog_manifest_to_artifacts,
     get_upstream_artifacts, get_release_props,
-    add_checksums_to_artifacts, add_release_props_to_artifacts,
-    get_task_bucket, get_task_action, validate_bucket_paths,
+    add_checksums_to_artifacts, get_task_bucket, get_task_action, validate_bucket_paths,
+    get_updated_buildhub_artifact
 )
 from beetmoverscript.utils import (
     get_hash, generate_beetmover_manifest,
     get_size, alter_unpretty_contents, matches_exclude,
-    get_candidates_prefix, get_releases_prefix, get_creds, get_bucket_name,
+    get_candidates_prefix, get_releases_prefix, get_creds,
+    get_bucket_name, get_bucket_url_prefix,
     is_release_action, is_promotion_action, get_partials_props,
-    get_product_name
+    get_product_name, is_partner_action, is_partner_private_task,
+    is_partner_public_task, write_json
 )
 
 log = logging.getLogger(__name__)
@@ -49,7 +56,7 @@ async def push_to_nightly(context):
     # determine artifacts to beetmove
     context.artifacts_to_beetmove = get_upstream_artifacts(context)
 
-    context.release_props, release_props_file = get_release_props(context)
+    context.release_props = get_release_props(context)
 
     # generate beetmover mapping manifest
     mapping_manifest = generate_beetmover_manifest(context)
@@ -86,10 +93,24 @@ async def push_to_nightly(context):
     # determine the correct checksum filename and generate it, adding it to
     # the list of artifacts afterwards
     add_checksums_to_artifacts(context)
-    # add release props file to later be used by beetmover jobs than upload
-    # the checksums file
-    if release_props_file:
-        add_release_props_to_artifacts(context, release_props_file)
+
+
+# push_to_partner {{{1
+async def push_to_partner(context):
+    """Push private repack artifacts to a certain location. They can be either
+    private (to private S3 buckets) or public (going under regular firefox
+    bucket).
+
+    Determine the list of artifacts to be transferred, generate the
+    mapping manifest and upload the bits."""
+    context.artifacts_to_beetmove = get_upstream_artifacts(context, preserve_full_paths=True)
+    context.release_props = get_release_props(context)
+    context.checksums = dict()
+
+    mapping_manifest = generate_beetmover_manifest(context)
+    await move_partner_beets(context, mapping_manifest)
+
+    add_checksums_to_artifacts(context)
 
 
 # push_to_releases {{{1
@@ -135,6 +156,49 @@ async def push_to_releases(context):
     copy_beets(context, candidates_keys_checksums, releases_keys_checksums)
 
 
+async def push_to_maven(context):
+    """Push artifacts to locations expected by maven clients (like mvn or gradle)"""
+    artifacts_to_beetmove = task.get_upstream_artifacts_with_zip_extract_param(context)
+    context.release_props = get_release_props(context)
+    context.checksums = dict()  # Needed by downstream calls
+    context.raw_balrog_manifest = dict()    # Needed by downstream calls
+
+    mapping_manifest = generate_beetmover_manifest(context)
+    validate_bucket_paths(context.bucket, mapping_manifest['s3_bucket_path'])
+
+    context.artifacts_to_beetmove = _extract_and_check_maven_artifacts_to_beetmove(
+        artifacts_to_beetmove,
+        mapping_manifest,
+        context.config.get('zip_max_file_size_in_mb', DEFAULT_ZIP_MAX_FILE_SIZE_IN_MB)
+    )
+
+    await move_beets(context, context.artifacts_to_beetmove, mapping_manifest)
+
+
+def _extract_and_check_maven_artifacts_to_beetmove(artifacts, mapping_manifest, zip_max_file_size_in_mb):
+    expected_files = maven_utils.get_maven_expected_files_per_archive_per_task_id(
+        artifacts, mapping_manifest
+    )
+
+    extracted_paths_per_archive = zip.check_and_extract_zip_archives(
+        artifacts, expected_files, zip_max_file_size_in_mb
+    )
+
+    number_of_extracted_archives = len(extracted_paths_per_archive)
+    if number_of_extracted_archives == 0:
+        raise ScriptWorkerTaskException('No archive extracted')
+    elif number_of_extracted_archives > 1:
+        raise NotImplementedError('More than 1 archive extracted. Only 1 is supported at once')
+    extracted_paths_per_relative_path = list(extracted_paths_per_archive.values())[0]
+
+    return {
+        'en-US': {
+            os.path.basename(path_in_archive): full_path
+            for path_in_archive, full_path in extracted_paths_per_relative_path.items()
+        }
+    }
+
+
 # copy_beets {{{1
 def copy_beets(context, from_keys_checksums, to_keys_checksums):
     creds = get_creds(context)
@@ -193,10 +257,12 @@ def list_bucket_objects(context, s3_resource, prefix):
 
 # action_map {{{1
 action_map = {
+    'push-to-partner': push_to_partner,
     'push-to-nightly': push_to_nightly,
     # push to candidates is at this point identical to push_to_nightly
     'push-to-candidates': push_to_nightly,
-    'push-to-releases': push_to_releases
+    'push-to-releases': push_to_releases,
+    'push-to-maven': push_to_maven,
 }
 
 
@@ -229,10 +295,36 @@ async def async_main(context):
 # move_beets {{{1
 async def move_beets(context, artifacts_to_beetmove, manifest):
     beets = []
+
     for locale in artifacts_to_beetmove:
+
+        installer_path = ''
+        buildhub_artifact_exists = False
+        # get path of installer beet
+        for artifact in artifacts_to_beetmove[locale]:
+            if artifact in INSTALLER_ARTIFACTS:
+                installer_path = artifacts_to_beetmove[locale][artifact]
+            if artifact == BUILDHUB_ARTIFACT:
+                buildhub_artifact_exists = True
+
+        # throws error if buildhub.json is present and installer isn't
+        if not installer_path and buildhub_artifact_exists:
+            raise ScriptWorkerTaskException(
+                "could not determine installer path from task payload"
+            )
+
+        # move beets
         for artifact in artifacts_to_beetmove[locale]:
             source = artifacts_to_beetmove[locale][artifact]
             artifact_pretty_name = manifest['mapping'][locale][artifact]['s3_key']
+
+            # update buildhub.json file
+            # if there is no installer then there will be no buildhub.json artifact
+            # in logical coding terms, this means that if installer_path is an empty
+            # string, then this if-block is never reached
+            if artifact == BUILDHUB_ARTIFACT:
+                write_json(source, get_updated_buildhub_artifact(source, installer_path, context, manifest, locale))
+
             destinations = [os.path.join(manifest["s3_bucket_path"],
                                          dest) for dest in
                             manifest['mapping'][locale][artifact]['destinations']]
@@ -283,12 +375,94 @@ async def move_beet(context, source, destinations, locale,
                                                                                    locale, destinations, from_buildid))
 
 
+# move_partner_beets {{{1
+async def move_partner_beets(context, manifest):
+    artifacts_to_beetmove = context.artifacts_to_beetmove
+    beets = []
+
+    for locale in artifacts_to_beetmove:
+        for full_path_artifact in artifacts_to_beetmove[locale]:
+            source = artifacts_to_beetmove[locale][full_path_artifact]
+            destination = get_destination_for_partner_repack_path(context, manifest,
+                                                                  full_path_artifact, locale)
+            beets.append(
+                asyncio.ensure_future(
+                    upload_to_s3(context=context, s3_key=destination, path=source)
+                )
+            )
+
+            if is_partner_public_task(context):
+                # we trim the full destination to the part after
+                # candidates/{version}-candidates/build{build_number}/
+                artifact_pretty_name = destination[destination.find(locale):]
+                if context.checksums.get(artifact_pretty_name) is None:
+                    context.checksums[artifact_pretty_name] = {
+                        algo: get_hash(source, algo) for algo in context.config['checksums_digests']
+                    }
+                    context.checksums[artifact_pretty_name]['size'] = get_size(source)
+
+    await raise_future_exceptions(beets)
+
+
+def sanity_check_partner_path(path, repl_dict, regexes):
+    for regex in regexes:
+        regex = regex.format(**repl_dict)
+        m = re.match(regex, path)
+        if m:
+            path_info = m.groupdict()
+            for substr in ("partner", "subpartner", "locale"):
+                if substr in regex and path_info[substr] in ('..', '.'):
+                    raise ScriptWorkerTaskException("Illegal partner path {} !".format(path))
+            # We're good.
+            break
+    else:
+        raise ScriptWorkerTaskException("Illegal partner path {} !".format(path))
+
+
+def get_destination_for_partner_repack_path(context, manifest, full_path, locale):
+    """Function to process the final destination path, relative to the root of
+    the S3 bucket. Depending on whether it's a private or public destination, it
+    performs several string manipulations.
+
+    Input: 'releng/partner/ghost/ghost-var/v1/linux-i686/ro/target.tar.bz2'
+    Possible ouput(s):
+        -> ghost/59.0b20-2/ghost-variant/v1/linux-i686/en-US/firefox-59.0b20.tar.bz2
+        -> pub/firefox/candidates/59.0b20-candidates/build2/partner-repacks/ghost/ghost-variant/v1/linux-i686/en-US/firefox-59.0b20.tar.bz2
+    """
+    # make sure we're calling this function from private-partner context
+    if not is_partner_action(context.action):
+        raise ScriptWorkerRetryException("Outside of private-partner context!")
+
+    # pretty name the `target` part to the actual filename
+    pretty_full_path = os.path.join(
+        locale,
+        manifest['mapping'][locale][os.path.basename(full_path)]
+    )
+
+    build_number = context.task['payload']['build_number']
+    version = context.task['payload']['version']
+
+    if is_partner_private_task(context):
+        sanity_check_partner_path(
+            locale, {'version': version, 'build_number': build_number},
+            PARTNER_REPACK_PRIVATE_REGEXES
+        )
+        return pretty_full_path
+    elif is_partner_public_task(context):
+        sanity_check_partner_path(
+            locale, {'version': version, 'build_number': build_number},
+            PARTNER_REPACK_PUBLIC_REGEXES
+        )
+        prefix = PARTNER_REPACK_PUBLIC_PREFIX_TMPL.format(version=version, build_number=build_number)
+        return os.path.join(prefix, pretty_full_path)
+
+
 # generate_balrog_info {{{1
 def generate_balrog_info(context, artifact_pretty_name, locale, destinations, from_buildid=None):
     release_props = context.release_props
     checksums = context.checksums
 
-    url = "{prefix}/{s3_key}".format(prefix="https://archive.mozilla.org",
+    url = "{prefix}/{s3_key}".format(prefix=get_bucket_url_prefix(context),
                                      s3_key=destinations[0])
 
     data = {
@@ -319,15 +493,13 @@ def enrich_balrog_manifest(context, locale):
                                  'http://download.cdn.mozilla.net/pub'])
 
     enrich_dict = {
-        "appName": get_product_name(release_props['appName'],
-                                    TEMPLATE_KEY_PLATFORMS[release_props['stage_platform']]),
+        "appName": get_product_name(release_props['appName'], release_props['stage_platform']),
         "appVersion": release_props["appVersion"],
         "branch": release_props["branch"],
         "buildid": release_props["buildid"],
         "extVersion": release_props["appVersion"],
         "hashType": release_props["hashType"],
         "locale": locale if not locale == 'multi' else 'en-US',
-        # XXX: temporary fix for devedition until we solve 1424482
         "platform": NORMALIZED_BALROG_PLATFORMS.get(release_props["stage_platform"],
                                                     release_props["stage_platform"]),
         "url_replacements": url_replacements
@@ -346,9 +518,6 @@ def enrich_balrog_manifest(context, locale):
 # retry_upload {{{1
 async def retry_upload(context, destinations, path):
     """Manage upload of `path` to `destinations`."""
-    # TODO rather than upload twice, use something like boto's bucket.copy_key
-    #   probably via the awscli subproc directly.
-    # For now, this will be faster than using copy_key() as boto would block
     uploads = []
     for dest in destinations:
         uploads.append(
@@ -378,7 +547,7 @@ async def put(context, url, headers, abs_filename, session=None):
 # upload_to_s3 {{{1
 async def upload_to_s3(context, s3_key, path):
     product = get_product_name(context.release_props['appName'].lower(),
-                               TEMPLATE_KEY_PLATFORMS[context.release_props['stage_platform']])
+                               context.release_props['stage_platform'])
     api_kwargs = {
         'Bucket': get_bucket_name(context, product),
         'Key': s3_key,
