@@ -7,27 +7,51 @@ import pytest
 from yarl import URL
 
 import beetmoverscript.script
-from beetmoverscript.script import (setup_mimetypes, put,
-                                    move_beets, move_beet, async_main,
-                                    main, enrich_balrog_manifest,
-                                    list_bucket_objects,
-                                    push_to_releases, copy_beets)
-from beetmoverscript.task import get_upstream_artifacts
+from beetmoverscript.script import (
+    async_main,
+    copy_beets,
+    enrich_balrog_manifest,
+    get_destination_for_partner_repack_path,
+    list_bucket_objects,
+    main,
+    move_beet,
+    move_beets,
+    move_partner_beets,
+    push_to_partner,
+    push_to_releases,
+    push_to_maven,
+    put,
+    sanity_check_partner_path,
+    setup_mimetypes,
+)
+from beetmoverscript.constants import (
+    PARTNER_REPACK_PRIVATE_REGEXES,
+    PARTNER_REPACK_PUBLIC_REGEXES,
+)
+from beetmoverscript.task import get_upstream_artifacts, get_release_props
 from beetmoverscript.test import (
-    context, get_fake_valid_config, get_fake_valid_task, get_fake_balrog_props,
-    noop_async
+    context, get_fake_valid_config, get_fake_valid_task,
+    noop_async, noop_sync, get_test_jinja_env,
 )
 from beetmoverscript.utils import generate_beetmover_manifest, is_promotion_action
 from scriptworker.context import Context
 from scriptworker.exceptions import (ScriptWorkerRetryException,
                                      ScriptWorkerTaskException)
-from scriptworker.test import event_loop, fake_session, fake_session_500
+from scriptworker.test import fake_session, fake_session_500
 
-assert event_loop, context  # silence flake8
+assert context  # silence flake8
 assert fake_session, fake_session_500  # silence flake8
 assert noop_async  # silence flake8
 
 
+# push_to_partner {{{1
+@pytest.mark.asyncio
+async def test_push_to_partner(context, mocker):
+    mocker.patch('beetmoverscript.script.move_partner_beets', new=noop_async)
+    mocker.patch('beetmoverscript.utils.JINJA_ENV', get_test_jinja_env())
+    await push_to_partner(context)
+
+
 # push_to_releases {{{1
 @pytest.mark.parametrize("candidates_keys,releases_keys,exception_type", ((
     {"foo.zip": "x", "foo.exe": "y"}, {}, None,
@@ -66,6 +90,54 @@ async def test_push_to_releases(context, mocker, candidates_keys,
         await push_to_releases(context)
 
 
+@pytest.mark.asyncio
+@pytest.mark.parametrize('extract_zip_output, ErrorRaised', ((
+    {
+        '/work_dir/cot/someTaskId/public/build/target.maven.zip': {
+            'org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar',  # noqa E501
+            'org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.md5': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.md5',  # noqa E501
+            'org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.sha1': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.sha1',    # noqa E501
+        },
+    },
+    None
+), (
+    {},
+    ScriptWorkerTaskException
+), (
+    {
+        '/work_dir/cot/someTaskId/public/build/target.maven.zip': {
+            'org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar',  # noqa E501
+        },
+        '/work_dir/cot/someOtherTaskId/public/build/target.maven.zip': {
+            'org/mozilla/geckoview-beta-armeabi-v7a/62.0b3/geckoview-beta-x86-62.0b3.aar': '/work_dir/cot/someOtherTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-armeabi-v7a/62.0b3/geckoview-beta-armeabi-v7a-62.0b3.aar',  # noqa E501
+        },
+    },
+    NotImplementedError
+)))
+async def test_push_to_maven(context, mocker, extract_zip_output, ErrorRaised):
+    mocker.patch('beetmoverscript.utils.JINJA_ENV', get_test_jinja_env())
+    context.task['payload']['upstreamArtifacts'] = []
+    mocker.patch('beetmoverscript.task.get_upstream_artifacts_with_zip_extract_param', new=lambda _: None)
+    mocker.patch('beetmoverscript.maven_utils.get_maven_expected_files_per_archive_per_task_id', new=lambda _, __: ('', {}))
+    mocker.patch('beetmoverscript.zip.check_and_extract_zip_archives', new=lambda _, __, ___: extract_zip_output)
+
+    if ErrorRaised is None:
+        async def assert_artifacts_to_beetmove(_, artifacts_to_beetmove, __):
+            assert artifacts_to_beetmove == {
+                'en-US': {
+                    'geckoview-beta-x86-62.0b3.aar': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar',  # noqa E501
+                    'geckoview-beta-x86-62.0b3.aar.md5': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.md5',  # noqa E501
+                    'geckoview-beta-x86-62.0b3.aar.sha1': '/work_dir/cot/someTaskId/public/build/target.maven.zip.out/org/mozilla/geckoview-beta-x86/62.0b3/geckoview-beta-x86-62.0b3.aar.sha1',    # noqa E501
+                },
+            }
+
+        mocker.patch('beetmoverscript.script.move_beets', new=assert_artifacts_to_beetmove)
+        await push_to_maven(context)
+    else:
+        with pytest.raises(ErrorRaised):
+            await push_to_maven(context)
+
+
 # copy_beets {{{1
 @pytest.mark.parametrize("releases_keys,raises", ((
     {}, False
@@ -155,26 +227,28 @@ def test_setup_mimetypes():
 
 
 # put {{{1
-def test_put_success(event_loop, fake_session):
+@pytest.mark.asyncio
+async def test_put_success(fake_session):
     context = Context()
     context.config = get_fake_valid_config()
     context.session = fake_session
-    response = event_loop.run_until_complete(
-        put(context, url=URL('https://foo.com/packages/fake.package'), headers={},
-            abs_filename='beetmoverscript/test/fake_artifact.json', session=fake_session)
+    response = await put(
+        context, url=URL('https://foo.com/packages/fake.package'), headers={},
+        abs_filename='beetmoverscript/test/fake_artifact.json', session=fake_session
     )
     assert response.status == 200
     assert response.resp == [b'asdf', b'asdf']
 
 
-def test_put_failure(event_loop, fake_session_500):
+@pytest.mark.asyncio
+async def test_put_failure(fake_session_500):
     context = Context()
     context.config = get_fake_valid_config()
     context.session = fake_session_500
     with pytest.raises(ScriptWorkerRetryException):
-        event_loop.run_until_complete(
-            put(context, url=URL('https://foo.com/packages/fake.package'), headers={},
-                abs_filename='beetmoverscript/test/fake_artifact.json', session=fake_session_500)
+        await put(
+            context, url=URL('https://foo.com/packages/fake.package'), headers={},
+            abs_filename='beetmoverscript/test/fake_artifact.json', session=fake_session_500
         )
 
 
@@ -187,12 +261,10 @@ def test_put_failure(event_loop, fake_session_500):
     "mozilla-beta", "push-to-releases",
 )))
 def test_enrich_balrog_manifest(context, branch, action):
-    context.release_props = get_fake_balrog_props()["properties"]
-    context.release_props['platform'] = context.release_props['stage_platform']
-    context.release_props['branch'] = branch
     context.task['payload']['build_number'] = 33
     context.task['payload']['version'] = '99.0b44'
     context.action = action
+    context.release_props['branch'] = branch
 
     expected_data = {
         'appName': context.release_props['appName'],
@@ -238,13 +310,16 @@ async def test_upload_to_s3(context, mocker):
 
 
 # move_beets {{{1
+@pytest.mark.asyncio
 @pytest.mark.parametrize("partials", (False, True))
-def test_move_beets(event_loop, partials):
+async def test_move_beets(partials, mocker):
+    mocker.patch('beetmoverscript.utils.JINJA_ENV', get_test_jinja_env())
+
     context = Context()
     context.config = get_fake_valid_config()
     context.task = get_fake_valid_task()
-    context.release_props = get_fake_balrog_props()["properties"]
-    context.release_props['platform'] = context.release_props['stage_platform']
+    context.release_props = context.task['payload']['releaseProperties']
+    context.release_props['stage_platform'] = context.release_props['platform']
     context.bucket = 'nightly'
     context.action = 'push-to-nightly'
     context.raw_balrog_manifest = dict()
@@ -265,6 +340,12 @@ def test_move_beets(event_loop, partials):
         os.path.abspath(
             'beetmoverscript/test/test_work_dir/cot/eSzfNqMZT_mSiQQXu8hyqg/public/build/target.test_packages.json'
         ),
+        os.path.abspath(
+            'beetmoverscript/test/test_work_dir/cot/eSzfNqMZT_mSiQQXu8hyqg/public/build/buildhub.json'
+        ),
+        os.path.abspath(
+            'beetmoverscript/test/test_work_dir/cot/eSzfNqMZT_mSiQQXu8hyqg/public/build/target.apk'
+        )
     ]
     expected_destinations = [
         ['pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake-99.0a1.en-US.target_info.txt',
@@ -275,6 +356,10 @@ def test_move_beets(event_loop, partials):
          'pub/mobile/nightly/latest-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.txt'],
         ['pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.test_packages.json',
          'pub/mobile/nightly/latest-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.test_packages.json'],
+        ['pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake-99.0a1.en-US.buildhub.json',
+         'pub/mobile/nightly/latest-mozilla-central-fake/en-US/fake-99.0a1.en-US.buildhub.json'],
+        ['pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake 99.0a1.en-US.target.apk',
+         'pub/mobile/nightly/latest-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.apk']
     ]
 
     expected_balrog_manifest = [
@@ -344,9 +429,7 @@ def test_move_beets(event_loop, partials):
             context.raw_balrog_manifest[locale][component].append(data)
 
     with mock.patch('beetmoverscript.script.move_beet', fake_move_beet):
-        event_loop.run_until_complete(
-            move_beets(context, context.artifacts_to_beetmove, manifest)
-        )
+        await move_beets(context, context.artifacts_to_beetmove, manifest)
 
     assert sorted(expected_sources) == sorted(actual_sources)
     assert sorted(expected_destinations) == sorted(actual_destinations)
@@ -358,13 +441,14 @@ def test_move_beets(event_loop, partials):
 
 
 # move_beet {{{1
+@pytest.mark.asyncio
 @pytest.mark.parametrize('update_manifest,action', [
      (True, 'push-to-candidates'),
      (True, 'push-to-nightly'),
      (False, 'push-to-nightly'),
      (False, 'push-to-candidates')
 ])
-def test_move_beet(event_loop, update_manifest, action):
+async def test_move_beet(update_manifest, action):
     context = Context()
     context.config = get_fake_valid_config()
     context.task = get_fake_valid_task()
@@ -388,11 +472,11 @@ def test_move_beet(event_loop, update_manifest, action):
         }
     ]
     context.action = action
+    context.bucket = 'nightly'
     context.checksums = dict()
     context.balrog_manifest = list()
     context.raw_balrog_manifest = dict()
-    context.release_props = get_fake_balrog_props()["properties"]
-    context.release_props['platform'] = context.release_props['stage_platform']
+    context.release_props = context.task['payload']['releaseProperties']
     locale = "sample-locale"
 
     target_source = 'beetmoverscript/test/test_work_dir/cot/eSzfNqMZT_mSiQQXu8hyqg/public/build/target.txt'
@@ -409,7 +493,7 @@ def test_move_beet(event_loop, update_manifest, action):
     expected_balrog_manifest = {
         'hash': '73b91c3625d70e9ba1992f119bdfd3fba85041e6f804a985a18efe06ebb1d4147fb044ac06b28773130b4887dd8b5b3bc63958e1bd74003077d8bc2a3909416b',
         'size': 18,
-        'url': 'https://archive.mozilla.org/pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.txt',
+        'url': 'https://archive.test/pub/mobile/nightly/2016/09/2016-09-01-16-26-14-mozilla-central-fake/en-US/fake-99.0a1.en-US.target.txt',
     }
     actual_upload_args = []
 
@@ -417,11 +501,9 @@ def test_move_beet(event_loop, update_manifest, action):
         actual_upload_args.extend([destinations, path])
 
     with mock.patch('beetmoverscript.script.retry_upload', fake_retry_upload):
-        event_loop.run_until_complete(
-            move_beet(context, target_source, target_destinations, locale,
-                      update_balrog_manifest=update_manifest,
-                      artifact_pretty_name=pretty_name, from_buildid=None)
-        )
+        await move_beet(context, target_source, target_destinations, locale,
+                        update_balrog_manifest=update_manifest,
+                        artifact_pretty_name=pretty_name, from_buildid=None)
     assert expected_upload_args == actual_upload_args
     if update_manifest:
         for k in expected_balrog_manifest.keys():
@@ -430,12 +512,10 @@ def test_move_beet(event_loop, update_manifest, action):
 
     expected_balrog_manifest['from_buildid'] = '19991231235959'
     with mock.patch('beetmoverscript.script.retry_upload', fake_retry_upload):
-        event_loop.run_until_complete(
-            move_beet(context, target_source, target_destinations, locale,
-                      update_balrog_manifest=update_manifest,
-                      artifact_pretty_name=pretty_name,
-                      from_buildid='19991231235959')
-        )
+        await move_beet(context, target_source, target_destinations, locale,
+                        update_balrog_manifest=update_manifest,
+                        artifact_pretty_name=pretty_name,
+                        from_buildid='19991231235959')
     if update_manifest:
         if is_promotion_action(context.action):
             expected_balrog_manifest['previousBuildNumber'] = '1'
@@ -445,6 +525,95 @@ def test_move_beet(event_loop, update_manifest, action):
                     expected_balrog_manifest[k])
 
 
+# move_partner_beets {{{1
+@pytest.mark.asyncio
+async def test_move_partner_beets(context, mocker):
+    context.artifacts_to_beetmove = get_upstream_artifacts(context, preserve_full_paths=True)
+    context.release_props = get_release_props(context)
+    mocker.patch('beetmoverscript.utils.JINJA_ENV', get_test_jinja_env())
+    mapping_manifest = generate_beetmover_manifest(context)
+
+    mocker.patch.object(beetmoverscript.script, 'get_destination_for_partner_repack_path', new=noop_sync)
+    mocker.patch.object(beetmoverscript.script, 'upload_to_s3', new=noop_async)
+    await move_partner_beets(context, mapping_manifest)
+
+
+# get_destination_for_partner_repack_path {{{1
+@pytest.mark.parametrize('full_path,expected,bucket,raises,locale', ((
+    'releng/partner/foobar/target.tar.bz2',
+    'ghost/9999.0-99/ghost-variant/linux-i686/en-US/firefox-9999.0.tar.bz2',
+    'dep-partner', False, 'ghost/9999.0-99/ghost-variant/linux-i686/en-US',
+), (
+    'releng/partner/ghost/ghost-variant/en-US/target.tar.bz2',
+    'pub/firefox/candidates/9999.0-candidates/build99/partner-repacks/ghost/ghost-variant/v1/linux-i686/en-US/firefox-9999.0.tar.bz2',
+    'dep', True, 'partner-repacks/ghost/ghost-variant/v1/linux-i686/en-US',
+), (
+    'releng/partner/ghost/ghost-variant/en-US/target.tar.bz2',
+    'pub/firefox/candidates/9999.0-candidates/build99/partner-repacks/ghost/ghost-variant/v1/linux-i686/en-US/firefox-9999.0.tar.bz2',
+    'dep', False, 'partner-repacks/ghost/ghost-variant/v1/linux-i686/en-US',
+)))
+def test_get_destination_for_partner_repack_path(context, full_path,
+                                                 expected, bucket, raises, locale):
+    context.bucket = bucket
+    context.action = 'push-to-partner'
+    context.task['payload']['build_number'] = 99
+    context.task['payload']['version'] = '9999.0'
+    context.task['payload']['releaseProperties'] = {
+      "appName": "Firefox",
+      "buildid": "20180328233904",
+      "appVersion": "9999.0",
+      "hashType": "sha512",
+      "platform": "linux",
+      "branch": "maple"
+    }
+    # hack in locale
+    for artifact_dict in context.task['payload']['upstreamArtifacts']:
+        artifact_dict['locale'] = locale
+    context.artifacts_to_beetmove = get_upstream_artifacts(context, preserve_full_paths=True)
+    context.release_props = get_release_props(context)
+    mapping_manifest = generate_beetmover_manifest(context)
+
+    if raises:
+        context.action = 'push-to-dummy'
+        with pytest.raises(ScriptWorkerRetryException):
+            get_destination_for_partner_repack_path(context, mapping_manifest,
+                                                    full_path, locale)
+    else:
+        assert expected == get_destination_for_partner_repack_path(context, mapping_manifest,
+                                                                   full_path, locale)
+
+
+# sanity_check_partner_path {{{1
+@pytest.mark.parametrize("path,raises,regexes", ((
+    "foo/bar", True, PARTNER_REPACK_PRIVATE_REGEXES
+), (
+    "foo/9999-1/bar/mac/baz", False, PARTNER_REPACK_PRIVATE_REGEXES
+), (
+    "../9999-1/bar/mac/baz", True, PARTNER_REPACK_PRIVATE_REGEXES
+), (
+    "foo/9999-1/../mac/baz", True, PARTNER_REPACK_PRIVATE_REGEXES
+), (
+    "foo/9999-1/bar/badplatform/baz", True, PARTNER_REPACK_PRIVATE_REGEXES
+), (
+    "mac-EME-free/foo", False, PARTNER_REPACK_PUBLIC_REGEXES
+), (
+    "badplatform-EME-free/foo", True, PARTNER_REPACK_PUBLIC_REGEXES
+), (
+    "partner-repacks/foo/foo-bar/v1/win32/en-US", False, PARTNER_REPACK_PUBLIC_REGEXES
+), (
+    "partner-repacks/foo/foo-bar/v1/badplatform/en-US", True, PARTNER_REPACK_PUBLIC_REGEXES
+), (
+    "partner-repacks/foo/foo-bar/v1/win32/en-US/extra", True, PARTNER_REPACK_PUBLIC_REGEXES
+)))
+def test_sanity_check_partner_path(path, raises, regexes):
+    repl_dict = {'version': '9999', 'build_number': 1}
+    if raises:
+        with pytest.raises(ScriptWorkerTaskException):
+            sanity_check_partner_path(path, repl_dict, regexes)
+    else:
+        sanity_check_partner_path(path, repl_dict, regexes)
+
+
 # async_main {{{1
 @pytest.mark.parametrize('action,raises', ((
     'push-to-nightly', False
@@ -458,6 +627,7 @@ async def test_async_main(context, mocker, action, raises):
     def fake_action(*args):
         return action
 
+    mocker.patch('beetmoverscript.utils.JINJA_ENV', get_test_jinja_env())
     mocker.patch('beetmoverscript.script.move_beets', new=noop_async)
     mocker.patch.object(beetmoverscript.script, 'get_task_action', new=fake_action)
     if raises:
@@ -471,7 +641,7 @@ async def test_async_main(context, mocker, action, raises):
 
 
 # main {{{1
-def test_main(event_loop, fake_session):
+def test_main(fake_session):
     context = Context()
     context.config = get_fake_valid_config()
 
